/***************************************************
Copyright 2003 Darrell Johnson 

This file is part of Yu-gi-oh.

Yu-gi-oh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yu-gi-oh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Yu-gi-oh; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************/

#include <stdlib.h>
#include <stdio.h>
#define MAGICTRAP 26
#define FIELDMAGIC 15

typedef struct{
	int level, atk, def;
   char *name, *type, *attribute, *color, *icon, *cat_num, *description, *card_desig;
}Card, *Card_p;

typedef struct{
	int numcards;		//number of cards in the deck
   Card **deck;		//the deck of cards
	Card *topcard;		//pointer to the top card in the deck
   int topcardpos;   //integer to the top card in the deck
	int maxcards;		//maximum number of cards that can be in the deck
}Deck, *Deck_p;

typedef struct{
	char* name;
	int lifepoints, decktype, win, loss;
	Deck_p entire_deck; 	//pointer to the deck of cards
   Deck_p side_deck;		//pointer to the side deck of cards
   Card_p *in_hand;		//pointer to the cards currently in the player's hand
   int ohp; 				//next open card position in hand
   int handmax;         //not really used i don't think
   int sdc_used[16];		//indicates that the card in the side deck has been used.
}Player, *Player_p;

typedef struct{
	Card_p card;
	int faced; 		//1 = face up, 0 = face down, -1 = no card there.
	int pos; 		//0 = defense mode, 1 = attack mode, -1 = no card there
   int set; 		//0 = can't anymore, 1 = can.  Only allowed to change monster from atk to def and vice versa once per turn.
   int equipped;  //0 if no equip magic card used, 1 if equip magic card is used
   int magic_atk; //extra atk points added from effects of magic or trap card
   int magic_def; //extra def points added from effects of magic or trap card
}Boardpos, *Boardpos_p;

typedef struct{
	Boardpos_p *position;//position 0 - 4 is the monster card zone
   							//position 5 - 9 is the magic/trap card zone
   int nomp;				//next open monster card position on the board
   int nomtp;				//next open magic/trap card position on the board
	Card_p fieldcard;
	Card_p* graveyard;
   int gy_pos;  			//counter for the grayeyard stack of cards
	Card_p fusion;
	Card_p deck;
}Board, *Board_p;

typedef struct{
	Player_p p1;     		//pointer to player 1
	Player_p p2;			//pointer to player 2
	Board_p b1;				//pointer to player 1's board
	Board_p b2;				//pointer to player 2's board
   Card_p blank;			//pointer to a blank card, used to indicate nothing is there
	int turn;            //indicates whose turn it is
	char phase;
   int attacking_monster;	//indicates the board position of the monster attacking
   int being_attacked;		//indicates the board position of the monster being attacked
	int step;
   int p1_field_card_active;
   int p2_field_card_active;
}Playing_Field, *Playing_Field_p;

extern int SetUpPlayer(void);
extern int SetUpOpponent(void);
extern void PrintInstructions(char*);
extern int LoadYugiDeck(Deck_p);
extern void SetUpBoards(void);
extern void ShuffleDeck(Deck_p);
extern void Shuffle(void);
extern void CreateSideDeck(void);
extern void PrepareToStart(void);
extern int Duel(void);
extern void FirstTurn(void);
extern void CardQuery(void);
extern void DrawPhase(Player_p);
extern void StandbyPhase(void);
extern int MainPhaseI(Player_p, Board_p);
extern void Reset(void);
extern void CompDrawPhase(Player_p);
extern int CompMainPhaseI(Player_p, Board_p);
extern void CompBattlePhase(Player_p, Board_p);
extern void CompDamageStep(int, int, int, Board_p, Player_p, int, int);
extern int BattlePhase(Player_p, Board_p);

extern void UseSideDeck(void);
extern void UpdateScreen(void);
extern int PlaceOnBoard(Card_p, Board_p, int, int, int);
extern int UpdateHand(Player_p, int);
extern int MoveToGraveyard(Card_p, Board_p, int, int);
extern int CheckMagicTrapPlay(int, int);
extern void CheckForMagicTrapEffects(void);
extern void PostBattleMagic(void);
extern int CheckTrapPlayable(Board_p);
extern int CheckCounterPlayable(Board_p);
extern int Attacking_Monster(Board_p);
extern int Check_Monster_To_Attack(Board_p);
extern int Monster_To_Attack(Board_p);
extern int DamageStep(Player_p, Board_p);
extern int getnumber(void);
extern int getletter(void);
extern int SummonHighLevel(int, Player_p, Board_p);
extern int CompSummonHighLevel(int, Player_p, Board_p);

extern void ArmoredGlass(void);
extern void CastleWalls(void);
extern void DragonCaptureJar(void);
extern int Earthshaker(void);
extern void Reinforcements(void);
extern void RespectPlay(void);
extern void ReverseTrap(void);
extern int TrapHole(void);
extern int TwoProngedAttack(void);
extern int UltimateOffering(void);
extern int Waboku(void);
extern void BookofSecretArts(void);
extern void CardDestruction(void);
extern int ChangeOfHeart(void);
extern int DarkHole(void);
extern int DeSpell(void);
extern void DianKeto(void);
extern int Fissure(void);
extern int LastWill(void);
extern int MonsterReborn(void);
extern int MonsterRecovery(void);
extern void RemoveTrap(void);
extern int SoulExchange(void);
extern void SwordDarkDestruct(void);
extern void Yami(void);
extern void InitMagicTrap(void);
extern int MagicAndTrapCards(char*);
extern void CheckStandbyPhase(void);
extern void CheckHands(void);
extern void HandToGraveyard(Player_p, Board_p, int);
extern void CheckFieldCard(void);

extern Playing_Field_p pf;			  	//this is the main playing field

int magic_trap_active[MAGICTRAP];	//this array is used to indicate there is
												//a magic or trap card active for player 1
int magic_trap_active2[MAGICTRAP];  //this array is used to indicate there is
												//a magic or trap card active for player 2
                                    //(when player 2 becomes able to use magic/trap cards.


