/***************************************************
Copyright 2003 Darrell Johnson 

This file is part of Yu-gi-oh.

Yu-gi-oh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yu-gi-oh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Yu-gi-oh; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <dos.h>

#include "datatypes3.h"

extern Playing_Field_p pf;

extern int magic_trap_active[MAGICTRAP];
extern int magic_trap_active2[MAGICTRAP];

char magic_trap_registry[MAGICTRAP][40] = {"Armored Glass", "Castle Walls", "Dragon Capture Jar",
"Earthshaker", "Reinforcements", "Respect Play", "Trap Hole", "Two-pronged Attack",
"Ultimate Offering", "Waboku", "Book of Secret Arts", "Card Destruction", "Change of Heart", "Dark Hole",
"De-Spell", "Dian Keto the Cure Master", "Fissure", "Last Will", "Monster Reborn",
"Monster Recovery", "Remove Trap", "Soul Exchange", "Sword of Dark Destruction", "Yami", "Reverse Trap"};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
////////////////////////////////Magic Section//////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

int MagicAndTrapCards(char* cardname){
   int i, found;
   i = found = 0;

	while(strcmp(magic_trap_registry[i], cardname) && i < MAGICTRAP){
   	i++;
      }

   if(i < MAGICTRAP) found = 1;
   //Some magic/trap cards result in the number of monsters on the field changing. (monsters destroyed or introduced onto the field)
   //Those magic cards will have a return value stored in found, and returned to calling function.
   //if found = 2 (the value returned when there is a new number of monsters on the field)
   //, this will trigger a Replay. found = 3 means skip the battle phase
   switch(i){
   	case 0:
      	ArmoredGlass();
         break;
      case 1:
      	CastleWalls();
         break;
      case 2:
      	DragonCaptureJar();
         break;
      case 3:
      	found = Earthshaker();
         break;
      case 4:
      	Reinforcements();
         break;
      case 5:
      	RespectPlay();
         break;
      case 6:
      	found = TrapHole();
         break;
      case 7:
      	found = TwoProngedAttack();
         break;
      case 8:
      	found = UltimateOffering();
         break;
      case 9:
      	found = Waboku();
         break;
      case 10:
      	BookofSecretArts();
         break;
      case 11:
      	CardDestruction();
         break;
      case 12:
      	found = ChangeOfHeart();
         break;
      case 13:
      	found = DarkHole();
         break;
      case 14:
      	found = DeSpell();
         break;
      case 15:
      	DianKeto();
         break;
      case 16:
      	found = Fissure();
         break;
      case 17:
      	found = LastWill();
         break;
      case 18:
      	found = MonsterReborn();
         break;
      case 19:
      	found = MonsterRecovery();
         break;
      case 20:
      	RemoveTrap();
         break;
      case 21:
      	found = SoulExchange();
         break;
      case 22:
      	SwordDarkDestruct();
         break;
      case 23:
      	Yami();
         break;
      case 24:
      	ReverseTrap();
      	break;
      default:
      	printf("Card not yet defined.\n");
         break;
         }
	return found;
}

void InitMagicTrap(void){
	int i;
	for(i = 0;i<MAGICTRAP;i++){
   	magic_trap_active[i] = 0;
      magic_trap_active2[i] = 0;
   }
}

void ArmoredGlass(){
	int i;
	printf("Armored Glass trap card activated.  All equip magic cards are negated.");
   for(i=0;i<5;i++){
   	if(pf->b1->position[i]->equipped == 1){
      	pf->b1->position[i]->equipped = 0;
         }
   	if(pf->b2->position[i]->equipped == 1){
      	pf->b2->position[i]->equipped = 0;
         }
      }
   }

void CastleWalls(){
	int in1;
	printf("Castle Walls trap card activated.  Raises defense by 500 points on one monster for one turn.\n");
   printf("Which monster will this card be used on? ");
   fflush(stdin);
	in1 = getnumber();
   pf->b1->position[in1]->magic_def += 500;
   }

void DragonCaptureJar(){
	int i;
   printf("Dragon Capture Jar trap card activated.  All dragons remain in defense mode while card is on field.\n");
   magic_trap_active[2] = 1;  //indicates that dragon capture jar is continuous
   for(i=0;i<5;i++){
   	if(strcmp(pf->b1->position[i]->card->type, "dragon"))
      	pf->b1->position[i]->pos = 0;
      if(strcmp(pf->b2->position[i]->card->type, "dragon"))
			pf->b1->position[i]->pos = 0;
		}
   }

int Earthshaker(){
	int i, j, in1, in2, found;
   char types[20][20] = {"none", "Dragon", "Spellcaster", "Zombie", "Beast-Warrior", "Beast", "Winged-Beast", "Fiend", "Fairy", "Insect", "Dinosaur",
	"Reptile", "Fish", "Sea-Serpent", "Machine", "Thunder", "Aqua", "Pyro", "Rock", "Plant"};

   found = 1;
   printf("Earthshaker trap card activated.  Pick 2 attributes.  One of the two attributes will be randomly selected.\n");
   printf("All face-up opponents of that attribute will be destroyed.\n");
   printf("The available types are:\n");
   for(i=0;i<20;i++)
   	printf("%d. %s\n", i, types[i]);
   printf("Pick one of the above types: ");
   fflush(stdin);
   in1 = getnumber();
   printf("Pick another of the above types: ");
   fflush(stdin);
   in2 = getnumber();
   j = rand() % 2;
   if(j){
   	printf("All %s type opponent monsters will be destroyed.\n", types[in1]);
      for(i=0;i<5;i++)
      	if(pf->b2->position[i]->card->name != NULL && strcmp(pf->b2->position[i]->card->type, types[in1]) ){
         	printf("%s destroyed.\n");
            MoveToGraveyard(pf->b2->position[i]->card, pf->b2, 0, i);
            found = 2;
            }
   	}
   else{
   	printf("All %s type opponent monsters will be destroyed.\n", types[in2]);
      for(i=0;i<5;i++)
      	if(pf->b2->position[i]->card->name != NULL && strcmp(pf->b2->position[i]->card->type, types[in2])){
         	printf("%s destroyed.\n");
            MoveToGraveyard(pf->b2->position[i]->card, pf->b2, 0, i);
            found = 2;
            }
   	}
   return found;
   }

void Reinforcements(){
	int i;
	printf("Reinforcements trap card activated.  One monster's ATK points increase 500 points for one turn.\n");
   printf("Which monster will this card be used on? (0 - %d): ", pf->b1->nomp-1);
   fflush(stdin);
	i = getnumber();
   pf->b1->position[i]->magic_atk += 500;
   }

void RespectPlay(){
	int i;
   printf("Respect Play trap card activated.  Each player must show their hand while this card is active on the board.\n");
   magic_trap_active[4] = 1;  //continuous

   for(i=0;i<pf->p2->ohp;i++)
   	if(pf->p2->in_hand[i]->name != NULL)
   		printf("%d. %s\n", i, pf->p2->in_hand[i]->name);

   }

void ReverseTrap(){
	int i;
	printf("Reverse Trap trap card activated.  All increases and decreases to ATK and DEF points are reversed.\n");
   for(i=0;i<5;i++){
   	pf->b1->position[i]->magic_atk = 0;
      pf->b1->position[i]->magic_def = 0;
      pf->b2->position[i]->magic_atk = 0;
      pf->b2->position[i]->magic_def = 0;
      }
   }

int TrapHole(){
	int found = 1;
   printf("Trap Hole trap card activated.  If attacking monster has more than 1000 ATK points, it is destroyed.\n");
   if(pf->turn == 1){
   	printf("Comp-U-Dueler's %s is attacking.\n", pf->b2->position[pf->attacking_monster]->card->name);
   	if(pf->b2->position[pf->attacking_monster]->card->atk > 1000){
   		printf("%s's atk greater than 1000, therefore it is destroyed by Trap Hole.\n",
         	pf->b2->position[pf->attacking_monster]->card->name);
         printf("%s sent to graveyard.\n", pf->b2->position[pf->attacking_monster]->card->name);
			MoveToGraveyard(pf->b2->position[pf->attacking_monster]->card, pf->b2, 0, 0);
         found = 2;
      	}
      else{
      	printf("%s's atk lesser than 1000, therefore unaffected by Trap Hole.\n",
         	pf->b2->position[pf->attacking_monster]->card->name);
         }
      }
   else{
   	printf("%s's %s is attacking.\n",pf->p1->name, pf->b2->position[pf->attacking_monster]->card->name);
   	if(pf->b1->position[pf->attacking_monster]->card->atk > 1000){
   		printf("%s's atk greater than 1000, therefore it is destroyed by Trap Hole.\n",
         	pf->b1->position[pf->attacking_monster]->card->name);
			MoveToGraveyard(pf->b1->position[pf->attacking_monster]->card, pf->b2, 0, 0);
         found = 2;
      	}
      else{
      	printf("%s's atk lesser than 1000, therefore unaffected by Trap Hole.\n",
         	pf->b1->position[pf->attacking_monster]->card->name);
         }
      }
   return found;
   }

int TwoProngedAttack(void){
	int in1, in2, in3, found ;
   found = 1;
 	printf("Two-Pronged Attack trap card activated.  Select and destroy 2 of your monsters, and one of the opponents.\n");
   printf("Select the first monster: ");
   fflush(stdin);
   in1 = getnumber();
   while(in1 != -1 && (pf->b1->position[in1]->card->name == NULL) ){
   	printf("Invalid selection.\n");
      printf("Select the first monster, or -1 to not use this card: ");
      fflush(stdin);
   	in1 = getnumber();
   }
   if(in1 == -1) return found;

   printf("Select the second monster: ");
   fflush(stdin);
   in2 = getnumber();
   while(pf->b1->position[in2]->card->name == NULL){
   	printf("Invalid selection.\n");
      printf("Select the first monster, or -1 to not use this card: ");
      fflush(stdin);
   	in2 = getnumber();
   }
   if(in2 == -1) return found;

   printf("Now select one of the opponent's monsters: ");
   in3 = getnumber();
   while(pf->b2->position[in3]->card->name == NULL){
   	printf("Invalid selection.\n");
      printf("Select the first monster, or -1 to not use this card: ");
   	in3 = getnumber();
   }
   if(in3 == -1) return found;

   printf("%s and %s destroyed to kill %s.\n", pf->b1->position[in1]->card->name,
   	pf->b1->position[in2]->card->name, pf->b2->position[in3]->card->name);

   MoveToGraveyard(pf->b1->position[in1]->card, pf->b1, 0, in1);
   MoveToGraveyard(pf->b1->position[in2]->card, pf->b1, 0, in2);
   MoveToGraveyard(pf->b2->position[in3]->card, pf->b2, 0, in3);
   found = 2;
   printf("Successful.\n");

   return found;
	}

int UltimateOffering(void){
	int i, in2, loss, found;
	printf("Ultimate Offering trap card activated.  At the cost of 500 life points per monster, you may summon another monster.\n");
   loss = 500 * pf->b1->nomp-1;
   pf->p1->lifepoints -= loss;
   printf("Which monster would you like to summon or set from your hand: ");
   i = getnumber();
   if(pf->p1->in_hand[i]->atk == -1){
      	printf("Placing on magic/trap card space.\n");
         PlaceOnBoard(pf->p1->in_hand[i], pf->b1, 0, 0, 1);
         UpdateHand(pf->p1, i);
         }
   else{
    printf("Place in (a)ttack or (d)efense mode: ");
    in2 = getletter();
    fflush(stdin);
    if(in2 == 'a'){
   	printf("%s placed in attack mode\n", pf->p1->in_hand[i]->name);
      PlaceOnBoard(pf->p1->in_hand[i], pf->b1, 1, 1, 0);
      UpdateHand(pf->p1, i);
      }
    else{
   	printf("%s placed in defense mode\n", pf->p1->in_hand[i]->name);
      PlaceOnBoard(pf->p1->in_hand[i], pf->b1, 0, 0, 0);
      UpdateHand(pf->p1, i);
      }
    }
   found = 2;
   return found;
   }

int Waboku(void){
	printf("Waboku trap card activated.  Any damage inflicted by an opponent's monster is decreased to 0.\n");
   //return a number status to end battle phase
   return 3; //4 may change to something else
   }

void BookofSecretArts(void){
	int in1;
	printf("Book of Secret Arts magic card activated.\n");
   printf("A Spellcaster-Type monster equipped with this card increases its ATK and DEF by 300 points.\n");
   printf("Which monster would you like to equip? ");
   in1 = getnumber();
   while(strcmp(pf->b1->position[in1]->card->type, "Spellcaster") && in1 != -1){
   	printf("Choose a spellcaster-type monster, or -1 to quit: ");
	}
   if(in1 != -1){
   	pf->b1->position[in1]->equipped = 1;
   	pf->b1->position[in1]->magic_atk += 300;
   	pf->b1->position[in1]->magic_def += 300;
      }
   return;
   }

int ChangeOfHeart(void){
   int in1, in2, found;
   found = 1;
	printf("Change of Heart magic card activated.\n");
   printf("Select and control one opposing monster (regardless of position) on the field until the end of your turn.\n");
   printf("Which of the opponent's monsters would you like to control? ");
	in1 = getnumber();
   while(pf->b2->position[in1]->card->name == NULL || (in1 > 4) ){
      printf("Select one of the opponent's monsters to control: ");
      in1 = getnumber();
      }
   printf("%s controls %s atk %d/def %d for this turn.\n", pf->p1->name, pf->b2->position[in1]->card->name,
   	pf->b2->position[in1]->card->atk, pf->b2->position[in1]->card->def);

   if(pf->b2->position[1]->card->name == NULL){
   	printf("Comp-U-Dueler has no monsters to deflect an attack.  You may attack the opponent's life points directly.\n");
      pf->p2->lifepoints -= pf->b2->position[in1]->card->atk;
      printf("Comp-U-Dueler has %d life points remaining.\n", pf->p2->lifepoints);
      return found;
      }
   printf("Which of the computer's monsters would you like to attack? ");
   in2 = getnumber();
   while(pf->b2->position[in2]->card->name == NULL){
   	printf("Select a valid monster to attack: ");
      in2 = getnumber();
      }
   pf->b1->position[pf->b1->nomp]->card = pf->b2->position[in1]->card;
   pf->b2->position[in1]->card = pf->blank;
   pf->attacking_monster = pf->b1->nomp;
   pf->being_attacked = in2;
   DamageStep(pf->p1, pf->b1);
   if(pf->b1->position[pf->b1->nomp]->card->name != NULL)
   	pf->b2->position[in1]->card = pf->b1->position[pf->b1->nomp]->card;
   found = 2;
   return found;
	}

void CardDestruction(void){
	int i;
	printf("Card Destruction magic card activated.\n");
   printf("Both players must discard their entire hands and draw the same number of cards that they discarded from their respective Decks.\n");
   for(i = 0; i < pf->p1->ohp; i++){
   	pf->b1->graveyard[pf->b1->gy_pos] = pf->p1->in_hand[i];
      pf->b1->gy_pos++;
      pf->p1->in_hand[i] = pf->p1->entire_deck->deck[pf->p1->entire_deck->topcardpos];
      pf->p1->entire_deck->topcardpos++;
		}
   for(i=0; i < pf->p2->ohp; i++){
		pf->b2->graveyard[pf->b2->gy_pos] = pf->p2->in_hand[i];
      pf->b2->gy_pos++;
      pf->p2->in_hand[i] = pf->p2->entire_deck->deck[pf->p2->entire_deck->topcardpos];
      pf->p2->entire_deck->topcardpos++;
      }
   }

int DarkHole(void){
	int i, found = 2;
	printf("Dark Hole magic card activated.\n");
   printf("Destroys all monsters on the field.\n");
   for(i = 0; i < 5; i++){
   	if(pf->b1->position[i]->card->name != NULL)
   		MoveToGraveyard(pf->b1->position[i]->card, pf->b1, 0, i);
      if(pf->b2->position[i]->card->name != NULL)
      	MoveToGraveyard(pf->b2->position[i]->card, pf->b2, 0, i);
      }
   return found;
	}

int DeSpell(void){
	int in1, found = 1;
	printf("De-Spell magic card activated.\n");
   printf("Destroys 1 Magic Card on the field.  If this card's target is face-down, flip it face-up.\n");
   printf("If the card is a Magic Card, it is destroyed.  If not, it is returned to its face-down position.\n");
   printf("The flipped card is not activated.\n");
   printf("Which magic card would you like to de-spell? (5 - 9): ");
   in1 = getnumber();
   if(strcmp(pf->b2->position[in1]->card->attribute, "magic") )
   	MoveToGraveyard(pf->b2->position[in1]->card, pf->b2, 1, in1);
   else
   	printf("The card you selected wasn't a magic card.\n");
   return found;
	}

void DianKeto(void){
	printf("Dian Keto the Cure Master magic card activated.\n");
	printf("Increases your Life Points by 1000 points.\n");
   pf->p1->lifepoints += 1000;
	}

int Fissure(void){
	int i, found, lowest = 9999;

	printf("Fissure magic card activated.\n");
   printf("Destroys 1 opponent's face-up monster with the lowest ATK.\n");
   for(i=0;i<5;i++)
   	if(pf->b2->position[i]->card->atk < lowest && (pf->b2->position[i]->faced == 1) )
      	lowest = i;
   printf("%s sent to graveyard.\n", pf->b2->position[lowest]->card->name);
   MoveToGraveyard(pf->b2->position[lowest]->card, pf->b2, 0, i);
   found = 2;
   return found;
   }

int LastWill(void){
	int i, in1, summoned = 0, found = 1;
	printf("Last Will magic card activated.\n");
   printf("If a monster of yours is sent from the field to the Graveyard during the turn that you've played this card\n");
   printf("you can select a monster with an ATK of 1500 points or less from your Deck and play it as a Special Summon.\n");
   printf("Shuffle the Deck after playing the card.  This card is active for 1 turn only.\n");
   if(pf->step > 3){
   	if(pf->turn == 1 && pf->b1->position[pf->being_attacked]->card->name == NULL){
      	printf("Last Will activated.\n");
         printf("Choose a card with less than 1500 atk points to draw from your deck.\n");
         for(i = pf->p1->entire_deck->topcardpos; i < pf->p1->entire_deck->numcards; i++){
         	if(pf->p1->entire_deck->deck[i]->atk <= 1500){
            	printf("Select %s %d/%d on the board? (y/n)", i, pf->p1->entire_deck->deck[i]->name,
               	pf->p1->entire_deck->deck[i]->atk, pf->p1->entire_deck->deck[i]->def);
               in1 = getletter();
               if(in1 == 'y'){
               	found = 2;
                  summoned = 1;
               	PlaceOnBoard(pf->p1->entire_deck->deck[in1], pf->b1, 1, 1, 0);
                  pf->p1->entire_deck->deck[in1] = pf->blank;
   					printf("%s has drawn %s and placed on board in attack position.\n",
                  	pf->p1->name, pf->p1->in_hand[pf->p1->ohp-1]->name);
                  ShuffleDeck(pf->p1->entire_deck);
                  printf("Deck shuffled.\n");
                  break;
                  }
            	}
            }
         }
      if(!summoned) printf("Failed to use Last Will.\n");
   		//2 = battle step
      }
      else
      	magic_trap_active[17] = 1;
   return found;
	}

int MonsterReborn(void){
	int i, in1, in2, found = 1;
   Board_p gy;
	printf("Monster Reborn magic card activated.\n");
   printf("Select 1 Monster Card from either your opponent's or your own Graveyard and place it on the field under your control in Attack or Defense Position (face-up).\n");
   printf("  This is considered a Special Summon.\n");
   do{
   	printf("Which graveyard? (0 for your graveyard or 1 for the opponent's graveyard): ");
   	in1 = getnumber();
   }while(in1 != 0 && in1 != 1);
   if(in1) gy = pf->b2;
   else gy = pf->b1;
   for(i=0;i< gy->gy_pos;i++){
   	if(gy->graveyard[i]->name != NULL)
   		printf("%s %d/%d level %d\n", gy->graveyard[i]->name, gy->graveyard[i]->atk,
         	gy->graveyard[i]->def, gy->graveyard[i]->level);
   	}
   printf("Which card will you resurrect from the graveyard? (0 - %d): ", gy->gy_pos-1);
   in2 = getnumber();
   if(gy->graveyard[in2]->name != NULL){
   	PlaceOnBoard(gy->graveyard[in2], pf->b1, 1, 1, 0);
      found = 2;
      }
   else
   	printf("An error occurred.  Couldn't resurrect monster.\n");
   return found;
	}

int MonsterRecovery(void){
	int found = 1;
	printf("Monster Recovery magic card activated.\n");
   printf("Select 1 of your own monsters on your side of the field and combine it with your Deck.  \n");
   printf("At the same time, combine your hand and Deck together and shuffle the Deck.  \n");
   printf("Draw the same number of cards that were in your hand from the shuffled deck.\n");
   return found;
	}

void RemoveTrap(void){
	int in1;
	printf("Remove Trap magic card activated.\n");
   printf("Destroys 1 face-up Trap Card on the field.\n");
   printf("Pick a face up trap card: ");
   in1 = getnumber();
   if(pf->b2->position[in1]->faced == 0 && pf->b2->position[in1]->card->atk != -1){
   	printf("Invalid selection. You probably can't use this now.  Save for later.\n");
      return;
      }
   MoveToGraveyard(pf->b2->position[in1]->card, pf->b2, 1, in1);
	}

int SoulExchange(void){
	int found = 1;
	printf("Soul Exchange magic card activated.\n");
   printf("Select an opponent's monster and use it as a Tribute in place of one of your own.\n");
   printf("  You must skip your Battle Phase for the turn in which this card is activated.");
   return found;
	}

void SwordDarkDestruct(void){
	int in1;
	printf("Sword of Dark Destruction magic card activated.\n");
   printf("A Dark monster equipped with this card increases its ATK by 400 points and decreases its DEF by 200 points.\n");
   printf("Pick a Dark monster to equip: ");
   in1 = getnumber();
   while(strcmp(pf->b1->position[in1]->card->attribute, "dark")){
   	printf("Pick a Dark monster to equip: ");
   	in1 = getnumber();
      }
   pf->b2->position[in1]->magic_atk += 400;
   pf->b2->position[in1]->magic_def -= 200;
   pf->b2->position[in1]->equipped = 1;
	}

void Yami(void){
	int i;
	printf("Yami magic card activated, which is a field magic card.\n");
   printf("Increases the ATK and DEF of all Fiend and Spellcaster-Type monsters by 200 points.  \n");
   printf("Also decreases the ATK and DEF of all Fairy-Type monsters by 200 points.\n");
   for(i=0;i<5;i++){
    if(pf->b1->position[i]->card->name != NULL){
   	if(!strcmp(pf->b1->position[i]->card->type, "Fiend") ||
         !strcmp(pf->b1->position[i]->card->type, "Spellcaster")){
         pf->b1->position[i]->magic_atk += 200;
         pf->b1->position[i]->magic_def += 200;
         }
       if(!strcmp(pf->b1->position[i]->card->type, "Fairy")){
         pf->b1->position[i]->magic_atk -= 200;
         pf->b1->position[i]->magic_def -= 200;
         }
      }
     if(pf->b2->position[i]->card->name != NULL){
      if(!strcmp(pf->b2->position[i]->card->type, "Fiend") ||
         !strcmp(pf->b2->position[i]->card->type, "Spellcaster")){
         pf->b2->position[i]->magic_atk += 200;
         pf->b2->position[i]->magic_def += 200;
         }

      if(!strcmp(pf->b2->position[i]->card->type, "Fairy")){
         pf->b2->position[i]->magic_atk -= 200;
         pf->b2->position[i]->magic_def -= 200;
         }
     }
    }
	}

void PostBattleMagic(void){
		int i;
      i = 10;
      if(i == 99) printf("NO!");
      else printf("OK!");
   //post battle magic only applies to certain cards, so we check for those
   if(magic_trap_active[17]){
   	printf("Last Will activated.\n");
      MagicAndTrapCards(magic_trap_registry[17]);
      magic_trap_active[17] = 0;
      }
}
