/***************************************************
Copyright 2003 Darrell Johnson 

This file is part of Yu-gi-oh.

Yu-gi-oh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yu-gi-oh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Yu-gi-oh; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <dos.h>

#include "datatypes3.h"

Playing_Field_p pf;

extern int magic_trap_active[MAGICTRAP];
extern int magic_trap_active2[MAGICTRAP];

char magic_trap_registry[MAGICTRAP][40] = {"Armored Glass", "Castle Walls", "Dragon Capture Jar",
"Earthshaker", "Reinforcements", "Respect Play", "Trap Hole", "Two-pronged Attack",
"Ultimate Offering", "Waboku", "Book of Secret Arts", "Card Destruction", "Change of Heart", "Dark Hole",
"De-Spell", "Dian Keto the Cure Master", "Fissure", "Last Will", "Monster Reborn",
"Monster Recovery", "Remove Trap", "Soul Exchange", "Sword of Dark Destruction", "Yami", "Reverse Trap"};


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
/////////////////////Player Initializing Functions//////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

//---Initializes data structures for player 1 and loads default values
int SetUpPlayer(void){
	char input[50];
   int i, j, in1;
	//char deck;  //variable for choosing yugi or kaiba deck

	pf->p1 = (Player_p)malloc(sizeof(Player)); //malloc a struct for the player
	pf->p1->win = 0;
	pf->p1->loss = 0;
   pf->p1->handmax = 6;
	pf->p1->entire_deck = (Deck_p)malloc(sizeof(Deck));
   printf("Enter name: ");
   fgets(input, 50, stdin);
   if(strlen(input) > 256) printf("Invalid input.  Try a shorter name.");
   i = strlen(input);
   input[i-1] = '\0';
   pf->p1->name = (char*)malloc(i);
	strcpy(pf->p1->name, input);  //copy name into player struct
	printf("Loading startup deck of cards...\n");
	LoadYugiDeck(pf->p1->entire_deck);

   printf("Greetings %s.  Your current record is %d-%d.\n",
   	pf->p1->name, pf->p1->win, pf->p1->loss,
		pf->p1->entire_deck->numcards);

   //this is a blank card to indicate there is no card in a player's hand.
   //when a player plays a card, the card pointer for the player's hand is changed
   //	to this card until a new card is drawn in it's place
   pf->blank = (Card_p)malloc(sizeof(Card));

   printf("You have these %d cards in your dueling deck:\n", pf->p1->entire_deck->numcards);
   for(i = 0;i<pf->p1->entire_deck->numcards;i++){
		printf("%d. %s  atk %d / def %d\n",i+1, pf->p1->entire_deck->deck[i]->name,
      pf->p1->entire_deck->deck[i]->atk, pf->p1->entire_deck->deck[i]->def);
		}

   do{
   	printf("\nSelect a number to get more info on a card, or type 0 to start the match: ");
      j = getnumber();
      if(j < 0 || j > pf->p1->entire_deck->numcards)
      	printf("Invalid selection.\n");
      else{
      	j = j-1;

      	if(j == -1) continue;
      	printf("%s  atk %d / def %d\ntype: %s\nattribute: %s\nspecial icon: %s\n\Description %s\n",
      	pf->p1->entire_deck->deck[j]->name, pf->p1->entire_deck->deck[j]->atk,
      	pf->p1->entire_deck->deck[j]->def, pf->p1->entire_deck->deck[j]->type,
      	pf->p1->entire_deck->deck[j]->attribute, pf->p1->entire_deck->deck[j]->icon,
      	pf->p1->entire_deck->deck[j]->description);
         }
      }while(j!= -1);

   printf("Would you like to get a brief description of how the game is played? (y/n): " );
   in1 = getletter();
   fflush(stdin);
   if(in1 == 'y')
   	PrintInstructions("howtoplay.txt");

   return 1;
	}

//----loads default data for a computer opponent and loads initial values
int SetUpOpponent(void){
	//char deck;  //variable for choosing yugi or kaiba deck

	pf->p2 = (Player_p)malloc(sizeof(Player)); //malloc a struct for the player
	pf->p2->win = 0;
	pf->p2->loss = 0;
   pf->p2->handmax = 6;
	pf->p2->entire_deck = (Deck_p)malloc(sizeof(Deck));

   pf->p2->name = "Comp-u-Dueler";
   printf("(o\\/o) (o\\/o) (o\\/o) (o\\/o) (o\\/o) (o\\/o) (o\\/o) (o\\/o)\n");
	printf("Loading opponent and his startup deck of cards...\n");
	LoadYugiDeck(pf->p2->entire_deck);

   printf("Comp-U-Dueler loaded.\n");
   printf("(o\\/o) (o\\/o) (o\\/o) (o\\/o) (o\\/o) (o\\/o) (o\\/o) (o\\/o)\n");
   return 1;
	}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
///////////////////////////////Card Operations//////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

//----creates data structures and loads the yugi deck of cards.
int LoadYugiDeck(Deck_p carddeck){
  int i, j, k, numcards;
  char file_input[1024];
  char* arguments[10];
  FILE *fp;
  numcards = 0;
  fp = fopen("carddb2.txt", "r");  //open card database for reading

  //count the number of cards in the database
  while( (fgets(file_input, 1024, fp) != NULL) ) {
  	numcards++;
   }

  fseek(fp, 0L, SEEK_SET);  //reset file position
  carddeck->deck = (Card_p*)calloc(numcards, sizeof(Card));
  for(i=0;i<numcards;i++)
  	carddeck->deck[i] = (Card_p)malloc(sizeof(Card));

  k = 0;
  while( (fgets(file_input, 1024, fp) != NULL) ) {  //read each line of the card database until the end-of-file
  		j=0;
  		while(file_input[j] != '\n'){  //chomp the newline character from the end of the input, sure i wish i could use perl for this step
		   j++;
		   }

		file_input[j] = '\0';

		//separate each line into separate arguments by the tab character that separates them
   	arguments[0] = strtok(file_input, "\t");
   	for(i=1;i<10;i++)
   		arguments[i] = strtok(NULL, "\t");

		//copy info into variables
      carddeck->deck[k]->name = (char*)malloc(strlen(arguments[0])+1);
		strcpy(carddeck->deck[k]->name, arguments[0]);
      carddeck->deck[k]->type = (char*)malloc(strlen(arguments[1])+1);
   	strcpy(carddeck->deck[k]->type, arguments[1]);
      carddeck->deck[k]->attribute = (char*)malloc(strlen(arguments[2])+1);
   	strcpy(carddeck->deck[k]->attribute, arguments[2]);
      carddeck->deck[k]->color = (char*)malloc(strlen(arguments[3])+1);
   	strcpy(carddeck->deck[k]->color, arguments[3]);
      carddeck->deck[k]->icon = (char*)malloc(strlen(arguments[7])+1);
   	strcpy(carddeck->deck[k]->icon, arguments[7]);
      carddeck->deck[k]->cat_num = (char*)malloc(strlen(arguments[8])+1);
		strcpy(carddeck->deck[k]->cat_num, arguments[8]);
      carddeck->deck[k]->description = (char*)malloc(strlen(arguments[9])+1);
		strcpy(carddeck->deck[k]->description, arguments[9]);
		carddeck->deck[k]->level = atoi(arguments[4]);
		carddeck->deck[k]->atk = atoi(arguments[5]);
   	carddeck->deck[k]->def = atoi(arguments[6]);

		k++;
   	//printf("Made card %d\n", k);
  }

  fclose(fp);
  carddeck->topcard = carddeck->deck[0];
  carddeck->topcardpos = 0;
  carddeck->numcards = numcards;
  carddeck->maxcards = 32767;
return 1;
}

//calls functions that shuffle each player's deck of cards
void Shuffle(void){
	int i;
	printf("\n=====================================\n");
	printf("Shuffling deck...");
   ShuffleDeck(pf->p1->entire_deck);
   for(i=0;i<20000;i++){
   	printf("\\"); printf("\b");sleep(0.8);printf("\|"); printf("\b");sleep(0.8);
      printf("\/"); printf("\b");sleep(0.8);printf("\-"); printf("\b");sleep(0.8);
      }
   printf("\n");
   printf("Shuffling opponent's deck....");
   for(i=0;i<20000;i++){
   	printf("\\"); printf("\b");sleep(0.8);printf("\|"); printf("\b");sleep(0.8);
      printf("\/"); printf("\b");sleep(0.8);printf("\-"); printf("\b");sleep(0.8);
      }
   printf("\n");
   printf("\n=====================================\n");
	ShuffleDeck(pf->p2->entire_deck);
   ShuffleDeck(pf->p2->entire_deck);
   }

void ShuffleDeck(Deck_p d){
	int i, j, k, retry, num;
   int *card_order;
   Deck_p new_deck;

   //this number array represents the ordering of the shuffled cards
   card_order = (int*)malloc(d->numcards * sizeof(int));

   new_deck = (Deck_p)malloc(sizeof(Deck)); //malloc a temporary deck
   new_deck->deck = (Card_p*)calloc(d->numcards, sizeof(Card));
  for(i = d->topcardpos; i < d->numcards + d->topcardpos; i++)
  	new_deck->deck[i] = (Card_p)malloc(sizeof(Card));

   randomize();
   for(i = d->topcardpos; i < d->numcards + d->topcardpos; i++)  //reset the card order
   	card_order[i] = -1;
   for(i = d->topcardpos; i < d->numcards + d->topcardpos; i++){
   	do{
      	retry = 0;                  //generate random number and check that
      	num = rand() % d->numcards; //there are no duplicates
         for(j=0;j < d->numcards;j++){
         	if(num == card_order[j]){
            	retry = 1;
               break;
               }
            }
         }while (retry == 1);
      card_order[i] = num;		//sets the random number into place
      }

      //this part is similar to the card creation stage. we malloc space for the
      //temp card deck(new_deck), then reorder the original deck according to the ordering
      //of the card_order array.
   for(k = d->topcardpos; k < d->numcards + d->topcardpos; k++){
   	new_deck->deck[k]->name = (char*)malloc(strlen(d->deck[card_order[k]]->name)+1);
		strcpy(new_deck->deck[k]->name, d->deck[card_order[k]]->name);

      new_deck->deck[k]->type = (char*)malloc(strlen(d->deck[card_order[k]]->type)+1);
   	strcpy(new_deck->deck[k]->type, d->deck[card_order[k]]->type);

      new_deck->deck[k]->attribute = (char*)malloc(strlen(d->deck[card_order[k]]->attribute)+1);
   	strcpy(new_deck->deck[k]->attribute, d->deck[card_order[k]]->attribute);

      new_deck->deck[k]->color = (char*)malloc(strlen(d->deck[card_order[k]]->color)+1);
   	strcpy(new_deck->deck[k]->color, d->deck[card_order[k]]->color);

      new_deck->deck[k]->icon = (char*)malloc(strlen(d->deck[card_order[k]]->icon)+1);
   	strcpy(new_deck->deck[k]->icon, d->deck[card_order[k]]->icon);

      new_deck->deck[k]->cat_num = (char*)malloc(strlen(d->deck[card_order[k]]->cat_num)+1);
		strcpy(new_deck->deck[k]->cat_num, d->deck[card_order[k]]->cat_num);

      new_deck->deck[k]->description = (char*)malloc(strlen(d->deck[card_order[k]]->description)+1);
		strcpy(new_deck->deck[k]->description, d->deck[card_order[k]]->description);

		new_deck->deck[k]->level = d->deck[card_order[k]]->level;
		new_deck->deck[k]->atk = d->deck[card_order[k]]->atk;
   	new_deck->deck[k]->def = d->deck[card_order[k]]->def;
		}

      //here, we free the old space used by the previous deck, make new space,
      //and copy everything back into the deck from the temporary deck (new_deck).
      //we also free the space from the temp deck.
      for(k = d->topcardpos; k < d->numcards + d->topcardpos; k++){
      free(d->deck[k]->name);
      d->deck[k]->name = (char*)malloc(strlen(new_deck->deck[k]->name)+1);
      strcpy(d->deck[k]->name, new_deck->deck[k]->name);
      free(new_deck->deck[k]->name);

      free(d->deck[k]->type);
   	d->deck[k]->type = (char*)malloc(strlen(new_deck->deck[k]->type)+1);
		strcpy(d->deck[k]->type, new_deck->deck[k]->type);
      free(new_deck->deck[k]->type);

      free(d->deck[k]->attribute);
		d->deck[k]->attribute = (char*)malloc(strlen(new_deck->deck[k]->attribute)+1);
		strcpy(d->deck[k]->attribute, new_deck->deck[k]->attribute);
      free(new_deck->deck[k]->attribute);

   	free(d->deck[k]->color);
		d->deck[k]->color = (char*)malloc(strlen(new_deck->deck[k]->color)+1);
		strcpy(d->deck[k]->color, new_deck->deck[k]->color);
      free(new_deck->deck[k]->color);

   	free(d->deck[k]->icon);
		d->deck[k]->icon = (char*)malloc(strlen(new_deck->deck[k]->icon)+1);
		strcpy(d->deck[k]->icon, new_deck->deck[k]->icon);
      free(new_deck->deck[k]->icon);

		free(d->deck[k]->cat_num);
		d->deck[k]->cat_num = (char*)malloc(strlen(new_deck->deck[k]->cat_num)+1);
		strcpy(d->deck[k]->cat_num, new_deck->deck[k]->cat_num);
      free(new_deck->deck[k]->cat_num);

		free(d->deck[k]->description);
		d->deck[k]->description = (char*)malloc(strlen(new_deck->deck[k]->description)+1);
		strcpy(d->deck[k]->description, new_deck->deck[k]->description);
      free(new_deck->deck[k]->description);

		d->deck[k]->level = new_deck->deck[k]->level;
		d->deck[k]->atk = new_deck->deck[k]->atk;
   	d->deck[k]->def = new_deck->deck[k]->def;
		}
      d->topcard = d->deck[d->topcardpos];

      free(new_deck->deck);
      free(new_deck);
      free(card_order);
      //hopefully we got it all, except the important stuff of course
   }

void CreateSideDeck(void){
	int i, j, k;

   printf("\nOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoO\n");
   printf("\nFifteen random cards will be selected to make up your Side Deck.\n");
   printf("You can pull from this deck at anytime in the match\nas long as you ");
	printf("don't have more than 6 cards in your hand by the end of your turn.\n");
   printf("Press enter to continue.");
   printf("\nOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoOoO\n");
   getchar();

   pf->p1->side_deck = (Deck_p)malloc(sizeof(Deck));
   pf->p1->side_deck->deck = (Card_p*)calloc(15, sizeof(Card));
  	for(i=0;i<15;i++)
  		pf->p1->side_deck->deck[i] = (Card_p)malloc(sizeof(Card));

   for(k=pf->p1->entire_deck->numcards-1, j=14;k > pf->p1->entire_deck->numcards-16 ;k--, j--){
		pf->p1->side_deck->deck[j]->name = (char*)malloc(strlen(pf->p1->entire_deck->deck[k]->name));
		strcpy(pf->p1->side_deck->deck[j]->name, pf->p1->entire_deck->deck[k]->name);
      free(pf->p1->entire_deck->deck[k]->name);

      pf->p1->side_deck->deck[j]->type = (char*)malloc(strlen(pf->p1->entire_deck->deck[k]->type)+1);
   	strcpy(pf->p1->side_deck->deck[j]->type, pf->p1->entire_deck->deck[k]->type);
      free(pf->p1->entire_deck->deck[k]->type);

      pf->p1->side_deck->deck[j]->attribute = (char*)malloc(strlen(pf->p1->entire_deck->deck[k]->attribute)+1);
   	strcpy(pf->p1->side_deck->deck[j]->attribute, pf->p1->entire_deck->deck[k]->attribute);
      free(pf->p1->entire_deck->deck[k]->attribute);

      pf->p1->side_deck->deck[j]->color = (char*)malloc(strlen(pf->p1->entire_deck->deck[k]->color)+1);
   	strcpy(pf->p1->side_deck->deck[j]->color, pf->p1->entire_deck->deck[k]->color);
      free(pf->p1->entire_deck->deck[k]->color);

      pf->p1->side_deck->deck[j]->icon = (char*)malloc(strlen(pf->p1->entire_deck->deck[k]->icon)+1);
   	strcpy(pf->p1->side_deck->deck[j]->icon, pf->p1->entire_deck->deck[k]->icon);
      free(pf->p1->entire_deck->deck[k]->icon);

      pf->p1->side_deck->deck[j]->cat_num = (char*)malloc(strlen(pf->p1->entire_deck->deck[k]->cat_num)+1);
		strcpy(pf->p1->side_deck->deck[j]->cat_num, pf->p1->entire_deck->deck[k]->cat_num);
      free(pf->p1->entire_deck->deck[k]->cat_num);

      pf->p1->side_deck->deck[j]->description = (char*)malloc(strlen(pf->p1->entire_deck->deck[k]->description)+1);
		strcpy(pf->p1->side_deck->deck[j]->description, pf->p1->entire_deck->deck[k]->description);
      free(pf->p1->entire_deck->deck[k]->description);

		pf->p1->side_deck->deck[j]->level = pf->p1->entire_deck->deck[k]->level;
		pf->p1->side_deck->deck[j]->atk = pf->p1->entire_deck->deck[k]->atk;
   	pf->p1->side_deck->deck[j]->def = pf->p1->entire_deck->deck[k]->def;

      free(pf->p1->entire_deck->deck[k]);
		}

   	pf->p1->entire_deck->numcards-=15;
      pf->p1->side_deck->numcards = 15;
      pf->p1->side_deck->maxcards = 15;
      for(i=0;i<15;i++)
      	pf->p1->sdc_used[i] = 0;

	printf("\n$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$\n");
   printf("\n$$$You have these %d in your side deck$$$\n", pf->p1->side_deck->numcards);
   for(j=0;j < 15;j++){
		printf("%d. %s  atk %d / def %d\n", j, pf->p1->side_deck->deck[j]->name,
      pf->p1->side_deck->deck[j]->atk, pf->p1->side_deck->deck[j]->def);
		}
   printf("\nPress enter to continue\n");
   printf("\n$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$\n");
   getchar();

   //printf("Now making computer's side deck\n");
      //Now a side deck for the computer
   pf->p2->side_deck = (Deck_p)malloc(sizeof(Deck));
   pf->p2->side_deck->deck = (Card_p*)calloc(15, sizeof(Card));
  	for(i=0;i<15;i++)
  		pf->p2->side_deck->deck[i] = (Card_p)malloc(sizeof(Card));

   for(k=pf->p2->entire_deck->numcards-1, j=14;k > pf->p2->entire_deck->numcards-16 ;k--, j--){
		pf->p2->side_deck->deck[j]->name = (char*)malloc(strlen(pf->p2->entire_deck->deck[k]->name));
		strcpy(pf->p2->side_deck->deck[j]->name, pf->p2->entire_deck->deck[k]->name);
      free(pf->p2->entire_deck->deck[k]->name);

      pf->p2->side_deck->deck[j]->type = (char*)malloc(strlen(pf->p2->entire_deck->deck[k]->type)+1);
   	strcpy(pf->p2->side_deck->deck[j]->type, pf->p2->entire_deck->deck[k]->type);
      free(pf->p2->entire_deck->deck[k]->type);

      pf->p2->side_deck->deck[j]->attribute = (char*)malloc(strlen(pf->p2->entire_deck->deck[k]->attribute)+1);
   	strcpy(pf->p2->side_deck->deck[j]->attribute, pf->p2->entire_deck->deck[k]->attribute);
      free(pf->p2->entire_deck->deck[k]->attribute);

      pf->p2->side_deck->deck[j]->color = (char*)malloc(strlen(pf->p2->entire_deck->deck[k]->color)+1);
   	strcpy(pf->p2->side_deck->deck[j]->color, pf->p2->entire_deck->deck[k]->color);
      free(pf->p2->entire_deck->deck[k]->color);

      pf->p2->side_deck->deck[j]->icon = (char*)malloc(strlen(pf->p2->entire_deck->deck[k]->icon)+1);
   	strcpy(pf->p2->side_deck->deck[j]->icon, pf->p2->entire_deck->deck[k]->icon);
      free(pf->p2->entire_deck->deck[k]->icon);

      pf->p2->side_deck->deck[j]->cat_num = (char*)malloc(strlen(pf->p2->entire_deck->deck[k]->cat_num)+1);
		strcpy(pf->p2->side_deck->deck[j]->cat_num, pf->p2->entire_deck->deck[k]->cat_num);
      free(pf->p2->entire_deck->deck[k]->cat_num);

      pf->p2->side_deck->deck[j]->description = (char*)malloc(strlen(pf->p2->entire_deck->deck[k]->description)+1);
		strcpy(pf->p2->side_deck->deck[j]->description, pf->p2->entire_deck->deck[k]->description);
      free(pf->p2->entire_deck->deck[k]->description);

		pf->p2->side_deck->deck[j]->level = pf->p2->entire_deck->deck[k]->level;
		pf->p2->side_deck->deck[j]->atk = pf->p2->entire_deck->deck[k]->atk;
   	pf->p2->side_deck->deck[j]->def = pf->p2->entire_deck->deck[k]->def;

      free(pf->p2->entire_deck->deck[k]);
		}
      pf->p2->entire_deck->numcards-=15;
      pf->p2->side_deck->numcards = 15;
      pf->p2->side_deck->maxcards = 15;
   }
