/***************************************************
Copyright 2003 Darrell Johnson 

This file is part of Yu-gi-oh.

Yu-gi-oh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yu-gi-oh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Yu-gi-oh; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <dos.h>

#include "datatypes3.h"

extern Playing_Field_p pf;

extern int magic_trap_active[MAGICTRAP];
extern int magic_trap_active2[MAGICTRAP];

char magic_trap_registry[MAGICTRAP][40] = {"Armored Glass", "Castle Walls", "Dragon Capture Jar",
"Earthshaker", "Reinforcements", "Respect Play", "Trap Hole", "Two-pronged Attack",
"Ultimate Offering", "Waboku", "Book of Secret Arts", "Card Destruction", "Change of Heart", "Dark Hole",
"De-Spell", "Dian Keto the Cure Master", "Fissure", "Last Will", "Monster Reborn",
"Monster Recovery", "Remove Trap", "Soul Exchange", "Sword of Dark Destruction", "Yami", "Reverse Trap"};

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
///////////////////////////////Info Functions///////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

//---allows the player to get information about each card in the deck
void CardQuery(void){
	int i;
   do{
   	printf("\nSelect a number to get more info on a card, or type -1 to start the match: ");
   	i = getnumber();
      if(i == -1) continue;
      else if( (i >= 6) || (i < -2) )
      	printf("Invalid card number\n");
      else{
			printf("%s  atk %d / def %d level %d\ntype: %s\nattribute: %s\nspecial icon: %s\n\Description %s\n",
         pf->p1->in_hand[i]->name, pf->p1->in_hand[i]->atk,
			pf->p1->in_hand[i]->def, pf->p1->in_hand[i]->level, pf->p1->in_hand[i]->type,
         pf->p1->in_hand[i]->attribute, pf->p1->in_hand[i]->icon,
         pf->p1->in_hand[i]->description);
         }
      }while(i!= -1);
   }

//---prints a brief description of the rules of the game
void PrintInstructions(char* filename){
	FILE *fp;
   char file_input[1024];
   int i = 0;
   fp = fopen(filename, "r");  //open instructions file for reading
   while( (fgets(file_input, 1024, fp) != NULL) ) {
  		printf("%s", file_input);
      if(i == 5){
      printf("\nPress Enter...");
      	getchar();
         i = 0;
         }
      i++;
      }
   fclose(fp);
   }

//---prints the current status of the board
void UpdateScreen(void){
	int i;
	printf("\n%s has %d life points remaining\n", pf->p1->name, pf->p1->lifepoints);
   printf("%s has %d life points remaining\n", pf->p2->name, pf->p2->lifepoints);

   //positions 0 - 4 will represent the monster card zone
   //positions 5 - 9 will represent the trap and magic card zone

   printf("\n-----------------------------------------------\n");

   for(i=0; i<10; i++){
   	if(pf->b1->position[i]->faced == -1)
      	continue;
      else if(pf->b1->position[i]->faced == 0)
      	printf("%d . %s has card %s atk %d/ def %d in defense mode\n", i, pf->p1->name, pf->b1->position[i]->card->name,
      	pf->b1->position[i]->card->atk + pf->b1->position[i]->magic_atk,
         pf->b1->position[i]->card->def + pf->b1->position[i]->magic_def);
      else printf("%d . %s has card %s atk %d/ def %d in attack mode\n", i, pf->p1->name	, pf->b1->position[i]->card->name,
      	pf->b1->position[i]->card->atk + pf->b1->position[i]->magic_atk,
         pf->b1->position[i]->card->def + pf->b1->position[i]->magic_def);
		}

   printf("\n-----------------------------------------------\n");
   if(pf->p1_field_card_active == 1) printf("Field card %s in play.\n%s\n", pf->b1->fieldcard->name, pf->b1->fieldcard->description);
   else if (pf->p2_field_card_active == 1) printf("Field card %s in play.\n%s\n", pf->b2->fieldcard->name, pf->b2->fieldcard->description);
   printf("\n-----------------------------------------------\n");
	for(i=0; i<10; i++){
   	if(pf->b2->position[i]->faced == -1)
      	continue;
      else if(pf->b2->position[i]->faced == 0)
      	printf("%d . faced-down card in defense mode\n", i, pf->p2->name, pf->b2->position[i]->card->name,
      	pf->b2->position[i]->card->atk, pf->b2->position[i]->card->def);
      else if(pf->b2->position[i]->faced == 1 && pf->b2->position[i]->pos == 0)
      	printf("%d. %s faced-up card in defense mode\n", i, pf->p2->name, pf->b2->position[i]->card->name,
      	pf->b2->position[i]->card->atk + pf->b2->position[i]->magic_atk,
         pf->b2->position[i]->card->def + pf->b2->position[i]->magic_def);
      else printf("%d . %s has card %s atk %d/ def %d in attack mode\n", i, pf->p2->name, pf->b2->position[i]->card->name,
      	pf->b2->position[i]->card->atk + pf->b2->position[i]->magic_atk,
         pf->b2->position[i]->card->def + pf->b2->position[i]->magic_def);
		}
   printf("\n-----------------------------------------------\n");
   sleep(2);
   }


//-------This function handles switching cards from the side deck to the player's hand.
//A player may move cards from the side deck to their hand, but not from the hand
//to the side deck, yet.  And a player can draw as many cards as they want during their turn,
//but if they have more than 6 cards in their hand by the end of the turn, excess cards must be
//discarded to the graveyard
void UseSideDeck(void){
	Player_p p = pf->p1;
	int i, choice, choice2;
   printf("\n----------//////////SIDE DECK\\\\\\\\\\------------\n");

	for(i=0; i<15; i++)
   	if(!pf->p1->sdc_used[i]){
      	printf("------------------------------------\n");
   		printf("%d. %s (atk %d / def %d) || level: %d || type: %s || attribute: %s\n", i, p->side_deck->deck[i]->name, p->side_deck->deck[i]->atk,
      	p->side_deck->deck[i]->def, p->side_deck->deck[i]->level, p->side_deck->deck[i]->type, p->side_deck->deck[i]->attribute);
      	printf("%s\n", p->side_deck->deck[i]->description);
      	//printf("------------------------------------\n");
         }
   choice = choice2 = 0;
   while(choice != 'x'){
   	printf("Would you like to (p)ull a card from your side deck, (l)ookup card info, or e(x)it? ");
      choice = getletter();
      if(choice == 'l')
      	CardQuery();
      if(choice == 'p'){
      	printf("Which card do you want to put in your deck?\n");
         printf("Keep in mind that you can't have more than 6 cards in your hand by the end of your turn ");
         fflush(stdin);
         choice2 = getnumber();

      	//if(pf->p1->ohp > pf->p1->handmax){
      	//	printf("Too many cards in your hand.\n");
         //	break;
         //	}
         if(pf->p1->sdc_used[choice2]){
         	printf("This card was already drawn once before.\n");
            break;
            }
        	pf->p1->in_hand[pf->p1->ohp] = pf->p1->side_deck->deck[choice2];
			pf->p1->sdc_used[choice2] = 1;
   		pf->p1->ohp++;

   		printf("%s has drawn %s.\n",pf->p1->name, pf->p1->in_hand[pf->p1->ohp-1]->name);
         }
      }
	}


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////Utility Functions///////////////////////////////////
/////////////////////////(redundant tasks in the game)//////////////////////////
////////////////////////////////////////////////////////////////////////////////

//----this functions handles placing a card on the board and updating positioning variables
int PlaceOnBoard(Card_p card, Board_p board, int faced, int pos, int cardtype){ //**pos indicates attack or defense mode
   int i, in1;

   if(!strcmp(card->icon, "field") ){
   	printf("Field card %s being introduced.\n", card->name);
      if(pf->b1->fieldcard->name != NULL){
      	pf->b1->graveyard[pf->b1->gy_pos] = pf->b1->fieldcard;
         pf->b1->gy_pos++;
         }
      if(pf->b2->fieldcard->name != NULL){
      	pf->b2->graveyard[pf->b2->gy_pos] = pf->b2->fieldcard;
         pf->b2->gy_pos++;
         }
      board->fieldcard = card;
      pf->p1_field_card_active = 1;

   	return 1;
      }

	if(cardtype == 0){	//monster card
   	if(board->nomp > 5){  //check monster card limit (should never really be needed during actual gameplay, but you never know
      	printf("Monster card limit reached.  You must select a card on the board to send to the graveyard.\n");
			for(i = 0; i < 5; i++){
         	printf("%d. %s %d / %d\n", i, board->position[i]->card->name,
            board->position[i]->card->atk, board->position[i]->card->def);
            }
         printf("Which card will you send to the graveyard? (0 - 5): ");
         fflush(stdin);
			in1 = getnumber();
         fflush(stdin);
         printf("Sending %s to graveyard.\n");
         MoveToGraveyard(board->position[in1]->card, board, cardtype, in1);
         }
   	board->position[board->nomp]->card = card;
      board->position[board->nomp]->faced = faced;
      board->position[board->nomp]->pos = pos;
      board->nomp++;
      }
   else if(cardtype == 1){
   	if(board->nomtp > 10){
      	printf("Trap/Magic card limit reached.  You must select a card on the board to send to the graveyard.\n");
			for(i = 5; i < 9; i++){
         	printf("%d. %s %d / %d\n", i, board->position[i]->card->name,
            board->position[i]->card->atk, board->position[i]->card->def);
            }
         printf("Which card will you send to the graveyard? (0 - 5): ");
         fflush(stdin);
         in1 = getnumber();
         fflush(stdin);
         printf("Sending %s to graveyard.\n");
         MoveToGraveyard(board->position[in1]->card, board, cardtype, in1);
         }
		board->position[board->nomtp]->card = card;
      board->position[board->nomtp]->faced = faced;
      board->position[board->nomtp]->pos = pos;
      board->nomtp++;
      }

	return 1;
}

//---this function is used to update a player's hand after a card has been played.
//It basically just shifts the array over each time a card is played.
int UpdateHand(Player_p player, int pos){
	int i;
   if(pos >= player->ohp){
   	player->in_hand[player->ohp] = pf->blank;
      return 1;
      }
	for(i=pos;i<player->ohp;i++)
   	player->in_hand[i] = player->in_hand[i+1];

   player->in_hand[player->ohp] = pf->blank;
	player->ohp--;
   return 1;
   }

//---this function handles moving a card from the board to the graveyard
int MoveToGraveyard(Card_p card, Board_p board, int cardtype, int boardpos){
		int i;
      board->graveyard[board->gy_pos] = card;
      board->gy_pos++;
   	board->position[boardpos]->card = pf->blank;
      board->position[boardpos]->faced = -1;

      if(!cardtype){
         for(i=boardpos;i<board->nomp-1;i++){
         	board->position[i]->card = board->position[i+1]->card;
            board->position[i]->faced = board->position[i+1]->faced;
            }
         board->position[board->nomp-1]->card = pf->blank;
         board->position[board->nomp-1]->faced = -1;
         board->nomp--;
         }
      else{
      	for(i=boardpos;i<board->nomtp-1;i++){
         	board->position[i]->card = board->position[i+1]->card;
            board->position[i]->faced = board->position[i+1]->faced;
            }
         board->position[board->nomtp-1]->card = pf->blank;
         board->position[board->nomtp-1]->faced = -1;
         board->nomtp--;
      }
	return 1;
}

//---in some instances, a player must discard cards from their hand directly to
//the graveyard.  This function handles that
void HandToGraveyard(Player_p p, Board_p b, int card){
  	b->graveyard[b->gy_pos] = p->in_hand[card];
   b->gy_pos++;
   UpdateHand(p, card);
}

//---if a field card is active on the board, the effects of the fieldcard are
//applied at the beginning of every turn
void CheckFieldCard(void){
	if(pf->p1_field_card_active)
   	MagicAndTrapCards(pf->b1->fieldcard->name);
   else if(pf->p2_field_card_active)
   	MagicAndTrapCards(pf->b2->fieldcard->name);
}

                  //---This function resets magic power-up values to 0, and the variable
void Reset(void){ //that keeps track of whether or not a card has been changed from atk to def mode.
						//Players can only change the set parameter once per turn,
                  //so this reallows the card to change position from attack to defense for the next turn.
	int i;
   for(i=0;i<10;i++){
      	pf->b1->position[i]->set = 0;
         //  will have to check if continuous card is in effect
         pf->b1->position[i]->magic_atk = 0;
         pf->b1->position[i]->magic_def = 0;

      	pf->b2->position[i]->set = 0;
         pf->b2->position[i]->magic_atk = 0;
         pf->b2->position[i]->magic_def = 0;
      }
}

//---if a player has a magic or trap card on the board that is playable, this function
//asks the player whether or not they want to use a magic/trap card.
int CheckMagicTrapPlay(int trap_playable, int countertrap_playable){
	int i, in1, in2, retval;
   retval = -1;
   if(trap_playable || countertrap_playable){
		printf("Enter 't' to use a trap now, or press enter to continue: ");
      fflush(stdin);
      in1 = getchar();
      fflush(stdin);
      if(in1 == 't'){
      	for(i=5; i< pf->b1->nomtp;i++){
   			if( strcmp(pf->b1->position[i]->card->attribute, "trap") ||
              ( strcmp(pf->b1->position[i]->card->attribute, "magic") && strcmp(pf->b1->position[i]->card->icon, "quickplay") ) ||
                strcmp(pf->b1->position[i]->card->icon, "counter") ){
               	printf("%d. Play %s? (y/n)", i, pf->b1->position[i]->card->name );
                  in2 = getletter();
                  if(in2 == 'y'){
                  	retval = MagicAndTrapCards(pf->b1->position[i]->card->name);
                  	if(strcmp(pf->b1->position[i]->card->icon, "continuous")){
                  		printf("%s played and now going to Graveyard.\n", pf->b1->position[i]->card->name);
                  		MoveToGraveyard(pf->b1->position[i]->card, pf->b1, 1, i);
                     	}
                     }
                  //return retval;
                  }//close 3rd if
            }//close for
		}//close 2nd if
	}//close 1st if

   switch(retval){
   	case -1:
      	//printf("Error in trying to use the magic or trap card.  Your turn will continue as normal.\n");
         retval = 0;
         break;
      case 0:
      	printf("This card hasn't been implemented yet.  Your turn will continue as normal.\n");
         retval = 0;
         break;
      case 1:
      	printf("Magic/Trap card play successful");
         retval = 0;
         break;
      case 2:
      	printf("Replay triggered.  Repeating a step.\n");
         retval = 1;
         break;
      case 3:
      	printf("Battle Step skipped as a result of magic/trap card.\n");
         retval = 2;
         break;
      default:
      	printf("Some type of error. Shouldn't be here ever.\n");
         retval = -1;
         break;
         };
   return retval;
}

//--At the end of a player's turn, if a magic card has been activated and used, but
//isn't a continuous magic/trap card, it is sent to the graveyard.  This function may need
//to be changed.  Sometimes cards stay on the field longer than they should.
void CheckForMagicTrapEffects(void){
	int i, j;
   for(i=5;i<10;i++){
    if(pf->b1->position[i]->card->name != NULL){
   	for(j=0;j<MAGICTRAP;j++){
   		if(magic_trap_active[j]){
         	if(strcmp(pf->b1->position[i]->card->name, magic_trap_registry[i]) )
            	printf("%s continuous effect in play.\n", magic_trap_registry[i]);
            else MoveToGraveyard(pf->b1->position[i]->card, pf->b1, 1, i);
         }
      }
    }
   }
}

int getnumber(void){
	char tmp[4];
   fflush(stdin);
	fgets(tmp, 4, stdin);
   fflush(stdin);
   return atoi(tmp);
   }

int getletter(void){
	int tmp;
   fflush(stdin);
   tmp = getc(stdin);
   fflush(stdin);
   if(!isalpha(tmp) ){
   	printf("Try again: ");
      tmp = getletter();
      }
   return tmp;
   }

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//////////////////////Player 1 Gameplay Helper Functions////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

//---returns 1 if a player has a magic, trap, or quickplay card on the board
int CheckTrapPlayable(Board_p b){
	int i, trap_card_playable;
	trap_card_playable = 0;
   for(i=5; i< b->nomtp;i++)
     if(b->position[i]->card->name != NULL)
   	if( strcmp(b->position[i]->card->attribute, "trap") ||
        ( strcmp(b->position[i]->card->attribute, "magic") && strcmp(b->position[i]->card->icon, "quickplay") ) )
        	trap_card_playable = 1;
   return trap_card_playable;
   }

//---returns 1 if a player has a countertrap card playable.  The distinction between
//traps and countertraps and quickplays etc.... hasn't been emplemented yet, but it will be.
int CheckCounterPlayable(Board_p b){
   int i, counter_trap_playable;
	counter_trap_playable = 0;
   for(i=5; i< b->nomtp;i++)
    if(b->position[i]->card->name != NULL)
   	if( strcmp(b->position[i]->card->icon, "counter") )
      	counter_trap_playable = 1;
	return counter_trap_playable;
   }

//---this function is called when a player has to decide which card to attack with
int Attacking_Monster(Board_p b){
	int input1 = 'a';

   while(isdigit(input1) || (input1 > 6) ){
   	printf("Which card would you like to attack with? ");
      fflush(stdin);
   	input1 = getnumber();
      }

   if(input1 == -1) return -1;

   while( (input1 != -1) && b->position[input1]->card->atk == -1){
   	printf("Choose a card that isn't a magic card, or -1 to end battle phase.\n");
      fflush(stdin);
      input1 = getnumber();
   	fflush(stdin);
      }

   while(input1 != -1 && (b->position[input1]->pos == 0) && (b->position[input1]->faced == 0) ){
   	printf("Choose a card that isn't in defense mode, or -1 to end battle phase..\n");
      fflush(stdin);
      input1 = getnumber();
   	fflush(stdin);
      }

   if(input1 == -1) return -1;
   printf("You've selected to attack with %s ", b->position[input1]->card->name);
   printf("%d / %d\n", b->position[input1]->card->atk,  b->position[input1]->card->def);
   pf->attacking_monster = input1;
   return input1;
}

//---if a monster is on the board, a 1 is returned.  If there is no monster on the board,
//the opposing player can attack the lifepoints directly.
int Check_Monster_To_Attack(Board_p b){
	int i, monster_there = 0;
	for(i=0;i<5;i++){
   	if(b->position[i]->card->name != NULL){
   		monster_there = 1;
      	break;
      	}
   	}
   return monster_there;
}

//---this function handles the determining of which monster will be getting attacked.
//sorry for the redundant function names.
int Monster_To_Attack(Board_p b){
	int input2;
   printf("Which of the opponent's cards do you wish to attack? ");
   fflush(stdin);
   input2 = getnumber();
   fflush(stdin);
   while(input2 < 0){
   	printf("Which of the opponent's cards do you wish to attack? ");
      input2 = getnumber();
      fflush(stdin);
      }
   printf("You have chosen to attack %s ", b->position[input2]->card->name);
   pf->being_attacked = input2;
   printf("atk %d /def %d ", b->position[input2]->card->atk,  b->position[input2]->card->def);
   if(b->position[input2]->faced == 0)
   	printf("which is in defense mode.\n");
   else printf("which is in attack mode.\n");
   return input2;
   }

//---calculates the damages inflicted for each battle
int DamageStep(Player_p p, Board_p b){
	int battle_atk1, battle_atk2, battle_def2, result;
   battle_atk1 = pf->b1->position[pf->attacking_monster]->card->atk + pf->b1->position[pf->attacking_monster]->magic_atk;
   battle_atk2 = pf->b2->position[pf->being_attacked]->card->atk + pf->b2->position[pf->being_attacked]->magic_atk;
   battle_def2 = pf->b2->position[pf->being_attacked]->card->def + pf->b2->position[pf->being_attacked]->magic_def;

   	 //Damage Step
       //At this point, check magic_trap_active for magic or trap cards that alter damage inflicted
	if(pf->b2->position[pf->being_attacked]->pos == 0){  //if you're attacking a monster that is in defense mode
     	if(battle_atk1 > battle_def2){
   	   printf("%s destroyed and sent to opponent's graveyard.\n", pf->b2->position[pf->being_attacked]->card->name);
         printf("Card was in defense mode so opponent loses no life points.\n");
         MoveToGraveyard(pf->b2->position[pf->being_attacked]->card, pf->b2, 0, pf->being_attacked);
         }
       else if(battle_atk1 == battle_def2){
       	printf("Your attack was nullified by opponent's defense.  Both cards remain.\n");
         printf("Neither player loses life points.\n");
         }
       else {
       	printf("Opponent's defense was greater than your offense.  %s destroyed %s.\n",
         		 pf->b2->position[pf->being_attacked]->card->name, b->position[pf->attacking_monster]->card->name);
         printf("Your monster is sent to the graveyard.\n");
         result = (battle_def2 - battle_atk1);
         printf("%d points deducted from your life points.\n", result);
         result = (p->lifepoints - result);
         p->lifepoints = result;
         printf("%d life points remain.\n", result);
         MoveToGraveyard(b->position[pf->attacking_monster]->card, b, 0, pf->attacking_monster);
         }
       } //end if
   	 //if you're attacking a monster that is in attack mode
   else{
   	if(battle_atk1 > battle_atk2){
      	printf("%s destroyed and sent to opponent's graveyard.\n", pf->b2->position[pf->being_attacked]->card->name);
         result = (battle_atk1 - battle_atk2);
         printf("%d points deducted from opponent's life points.\n", result);
         result = pf->p2->lifepoints - result;
         pf->p2->lifepoints = result;
         printf("%d life points remain.\n", result);
         MoveToGraveyard(pf->b2->position[pf->being_attacked]->card, pf->b2, 0, pf->being_attacked);
         }
      else if(battle_atk1 == battle_atk2){
      	printf("Your attack was nullified by opponent's attack.  Both cards destroyed and sent to graveyard.\n");
         printf("Neither player loses life points.\n");
         MoveToGraveyard(pf->b2->position[pf->being_attacked]->card, pf->b2, 0, pf->being_attacked);
         MoveToGraveyard(b->position[pf->attacking_monster]->card, b, 0, pf->attacking_monster);
         }
      else {
        	printf("Opponent's attack points were greater than your attack points.  %s destroyed %s.\n",
        		 pf->b2->position[pf->being_attacked]->card->name, b->position[pf->attacking_monster]->card->name);
         printf("Your monster is sent to the graveyard.\n");
         result = (battle_atk2 - battle_atk1);
         printf("%d points deducted from your life points.\n", result);
         result = (p->lifepoints - result);
         p->lifepoints = result;
         printf("%d life points remain.\n", result);
         MoveToGraveyard(b->position[pf->attacking_monster]->card, b, 0, pf->attacking_monster);
         }
      }
   return 1;
	}

//--when summoning a monster that's higher than level 4, low level monsters currently on the
//board must be sacrificed, or offered as a tribute, to successfully summon the monster
int SummonHighLevel(int input1, Player_p p, Board_p b){
	int tryagain, in2, in3, success;
   success = 0;
	do{
   	tryagain = 0;
   	if(p->in_hand[input1]->level > 4){
   		if(p->in_hand[input1]->level > 6){
      		printf("Summoning a level 7 or higher monster requires a tribute of 2 monsters.\n");
         	printf("Do you wish to proceed? (y/n): ");
				in2 = getletter();
            fflush(stdin);
            }
         else{
         	printf("Summoning a level 5 or 6 monster requires a tribute of 1 monsters.\n");
         	printf("Do you wish to proceed? (y/n): ");
				in2 = getletter();
            fflush(stdin);
            }
         }
      if(in2 == 'y'){
      	if( (p->in_hand[input1]->level > 6) && (b->position[1]->card->name == NULL) ||
         	 (p->in_hand[input1]->level > 4) && (b->position[0]->card->name == NULL) ){
             	printf("It appears that you don't have enough monsters to offer as a tribute now.\n");
               tryagain = 1;
               }
         else{
         	printf("Which monster would you like to offer as a tribute? (0-%d): ", b->nomp-1);
            in2 = getnumber();
            printf("Offering %s as a tribute to summon %s.\n",
               	b->position[in2]->card->name, p->in_hand[input1]->name);
            MoveToGraveyard(b->position[in2]->card, b, 0, in2);
            success = 1;
            if(p->in_hand[input1]->level > 6){
            	UpdateScreen();
            	printf("Choose a 2nd monster to offer as a tribute: ");
               in3 = getnumber();
               printf("Offering %s as a tribute to summon %s.\n",
               	b->position[in3]->card->name,
                  p->in_hand[input1]->name);
               MoveToGraveyard(b->position[in3]->card, b, 0, in3);
               success = 1;
               }
            else{


               }
				}
         }
      }while(tryagain == 1);
   return success;
}
